"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from .event_emitter import EventEmitter, EventEmitterProtocol, EventHandler

__all__ = ["EventEmitter", "EventHandler", "EventEmitterProtocol"]



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from enum import Enum


class ANSI(str, Enum):
    RESET = "\033[0m"

    BOLD = "\033[1m"
    BOLD_RESET = "\033[22m"
    ITALIC = "\033[3m"
    ITALIC_RESET = "\033[23m"
    UNDERLINE = "\033[4m"
    UNDERLINE_RESET = "\033[24m"
    STRIKE = "\033[9m"
    STRIKE_RESET = "\033[29m"

    FOREGROUND_RESET = "\033[0m"
    BACKGROUND_RESET = "\033[0m"
    FOREGROUND_BLACK = "\033[30m"
    BACKGROUND_BLACK = "\033[40m"
    FOREGROUND_RED = "\033[31m"
    BACKGROUND_RED = "\033[41m"
    FOREGROUND_GREEN = "\033[32m"
    BACKGROUND_GREEN = "\033[42m"
    FOREGROUND_YELLOW = "\033[33m"
    BACKGROUND_YELLOW = "\033[43m"
    FOREGROUND_BLUE = "\033[34m"
    BACKGROUND_BLUE = "\033[44m"
    FOREGROUND_MAGENTA = "\033[35m"
    BACKGROUND_MAGENTA = "\033[45m"
    FOREGROUND_CYAN = "\033[36m"
    BACKGROUND_CYAN = "\033[46m"
    FOREGROUND_WHITE = "\033[37m"
    BACKGROUND_WHITE = "\033[47m"
    FOREGROUND_GRAY = "\033[90m"
    FOREGROUND_DEFAULT = "\033[39m"
    BACKGROUND_DEFAULT = "\033[49m"



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

import functools
import inspect
import warnings
from typing import Any, Callable, Optional, TypeVar

F = TypeVar("F", bound=Callable[..., Any])


class ExperimentalWarning(FutureWarning):
    """Warning category for Teams SDK preview APIs.

    Preview APIs may change in the future.
    """

    pass


def experimental(diagnostic: str, *, message: Optional[str] = None) -> Callable[[F], F]:
    """Mark a class or function as a preview API.

    Emits an ExperimentalWarning when the decorated class is instantiated
    or the decorated function is called.

    Args:
        diagnostic: The diagnostic code (e.g., "ExperimentalTeamsReactions") for granular opt-in.
        message: Optional custom warning message. If not provided, a default message is used.

    Usage::

        @experimental("ExperimentalTeamsReactions")
        class ReactionClient:
            ...

        @experimental("ExperimentalTeamsTargeted", message="Targeted messages are in preview.")
        async def create_targeted(...):
            ...
    """

    def decorator(obj: F) -> F:
        name = getattr(obj, "__qualname__", getattr(obj, "__name__", str(obj)))
        warn_msg = message or (f"{name} is in preview and may change in the future. Diagnostic: {diagnostic}")

        if isinstance(obj, type):
            original_init = obj.__init__

            @functools.wraps(original_init)
            def new_init(self: Any, *args: Any, **kwargs: Any) -> None:
                warnings.warn(warn_msg, ExperimentalWarning, stacklevel=2)
                original_init(self, *args, **kwargs)

            obj.__init__ = new_init  # type: ignore[misc]
            return obj  # type: ignore[return-value]
        else:
            if inspect.iscoroutinefunction(obj):

                @functools.wraps(obj)
                async def async_wrapper(*args: Any, **kwargs: Any) -> Any:
                    warnings.warn(warn_msg, ExperimentalWarning, stacklevel=2)
                    return await obj(*args, **kwargs)

                return async_wrapper  # type: ignore[return-value]
            else:

                @functools.wraps(obj)
                def wrapper(*args: Any, **kwargs: Any) -> Any:
                    warnings.warn(warn_msg, ExperimentalWarning, stacklevel=2)
                    return obj(*args, **kwargs)

                return wrapper  # type: ignore[return-value]

    return decorator



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

import logging as _logging

from . import events, http, logging, storage  # noqa: E402
from .events import *  # noqa: F401, F402, F403
from .experimental import ExperimentalWarning, experimental
from .http import *  # noqa: F401, F402, F403
from .logging import *  # noqa: F401, F402, F403
from .storage import *  # noqa: F401, F402, F403

_logging.getLogger(__name__).addHandler(_logging.NullHandler())

# Combine all exports from submodules
__all__: list[str] = ["ExperimentalWarning", "experimental"]
__all__.extend(events.__all__)
__all__.extend(http.__all__)
__all__.extend(logging.__all__)
__all__.extend(storage.__all__)



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from collections import OrderedDict
from dataclasses import dataclass
from typing import Dict, List, Optional, TypeVar

from .storage import Storage

V = TypeVar("V")


@dataclass(frozen=True)
class LocalStorageOptions:
    max: Optional[int] = None
    """Maximum number of items in the storage"""


class LocalStorage(Storage[str, V]):
    """
    A key-value storage with optional size limit and LRU behavior.
    """

    @property
    def store(self) -> OrderedDict[str, V]:
        return self._store

    @property
    def options(self) -> LocalStorageOptions:
        return self._options

    @property
    def keys(self) -> List[str]:
        return list(self._store.keys())

    @property
    def size(self) -> int:
        return len(self._store)

    def __init__(
        self,
        data: Optional[Dict[str, V]] = None,
        options: Optional[LocalStorageOptions] = None,
    ):
        self._store = OrderedDict(data or {})
        self._options = options or LocalStorageOptions()

    def get(self, key: str) -> Optional[V]:
        if key not in self._store:
            return None

        value = self._store.pop(key)
        self._store[key] = value
        return value

    async def async_get(self, key: str) -> Optional[V]:
        return self.get(key)

    def set(self, key: str, value: V) -> None:
        if key in self._store:
            del self._store[key]
        elif self._options.max and len(self._store) >= self._options.max:
            self._store.popitem(last=False)

        self._store[key] = value

    async def async_set(self, key: str, value: V) -> None:
        return self.set(key, value)

    def delete(self, key: str) -> None:
        if key in self._store:
            del self._store[key]

    async def async_delete(self, key: str) -> None:
        return self.delete(key)



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from .client import Client, ClientOptions
from .client_token import Token, TokenFactory, resolve_token
from .interceptor import Interceptor, InterceptorRequestContext, InterceptorResponseContext

__all__ = [
    "Client",
    "ClientOptions",
    "Interceptor",
    "InterceptorRequestContext",
    "InterceptorResponseContext",
    "Token",
    "TokenFactory",
    "resolve_token",
]



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

import json
import logging

from .ansi import ANSI


class ConsoleFormatter(logging.Formatter):
    """
    A custom logging formatter that formats log messages with colors and prefixes.
    """

    _colors = {
        "ERROR": ANSI.FOREGROUND_RED,
        "WARNING": ANSI.FOREGROUND_YELLOW,
        "INFO": ANSI.FOREGROUND_CYAN,
        "DEBUG": ANSI.FOREGROUND_MAGENTA,
    }

    def format(self, record: logging.LogRecord) -> str:
        """
        Format the log record with colors and prefixes.
        Args:
            record (logging.LogRecord): The log record to format.
        Returns:
            str: The formatted log message.
        """
        if isinstance(record.msg, (dict, list)):
            record.msg = json.dumps(record.msg, indent=2)  # pyright: ignore[reportUnknownMemberType]

        level_name = record.levelname.upper()
        color = self._colors.get(level_name, ANSI.FOREGROUND_CYAN)
        prefix = f"{color.value}{ANSI.BOLD.value}[{level_name}]"
        name = f"{record.name}{ANSI.FOREGROUND_RESET.value}{ANSI.BOLD_RESET.value}"

        message = record.getMessage()

        # Include exception info if present
        if record.exc_info:
            message += "\n" + self.formatException(record.exc_info)

        lines = message.split("\n")
        formatted_lines = [f"{prefix} {name} {line}" for line in lines]
        return "\n".join(formatted_lines)



"""
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the MIT License.
"""

from .list_local_storage import ListLocalStorage
from .local_storage import LocalStorage, LocalStorageOptions
from .storage import ListStorage, Storage

__all__ = ["Storage", "ListStorage", "LocalStorage", "ListLocalStorage", "LocalStorageOptions"]
